/* $Id$

This file is part of gldbg
Copyright © 2010  Mikko Rasa, Mikkosoft Productions
Distributed under the GPL
*/

#include <dlfcn.h>
#include "arraysize.h"
#include "breakpoint.h"
#include "functions.h"
#include "glwrap.h"
#include "opengl.h"
#include "packet.h"

static int in_begin_block = 0;
static GLenum cur_error = GL_NO_ERROR;

static void check_error()
{
	static GLenum (*orig_glGetError)() = NULL;
	GLenum code;

	if(in_begin_block)
		return;

	if(!orig_glGetError)
		orig_glGetError = glsym("glGetError");

	code = orig_glGetError();
	if(code!=GL_NO_ERROR)
	{
		GlPacket *pkt;

		pkt = packet_begin(FUNC_GLDERROR);
		packet_write_int(pkt, code);
		packet_send(pkt, get_out_fd());

		if(cur_error==GL_NO_ERROR)
			cur_error = code;
	}
}

void APIENTRY glBegin(GLenum mode)
{
	static void (*orig)(GLenum) = NULL;
	GlPacket *pkt;

	if(!orig)
		orig = glsym("glBegin");
	orig(mode);

	pkt = packet_begin(FUNC_GLBEGIN);
	packet_write_int(pkt, mode);
	packet_send(pkt, get_out_fd());

	in_begin_block = 1;
}

void APIENTRY glEnd()
{
	static void (*orig)() = NULL;
	GlPacket *pkt;

	if(!orig)
		orig = glsym("glEnd");
	orig();

	pkt = packet_begin(FUNC_GLEND);
	packet_send(pkt, get_out_fd());

	in_begin_block = 0;
	check_error();
}

GLenum APIENTRY glGetError()
{
	GLenum ret = GL_NO_ERROR;
	GlPacket *pkt;

	if(in_begin_block)
	{
		if(cur_error==GL_NO_ERROR)
			cur_error = GL_INVALID_OPERATION;
	}
	else
	{
		ret = cur_error;
		cur_error = GL_NO_ERROR;
	}

	pkt = packet_begin(FUNC_GLGETERROR);
	packet_write_int(pkt, ret);
	packet_send(pkt, get_out_fd());

	return ret;
}

void (*glXGetProcAddress(const GLubyte *procname))(void)
{
	void *handle = 0;
	void (*ret)() = 0;
	GlPacket *pkt;

	if(glsym((const char *)procname))
	{
		handle = dlopen(NULL, RTLD_LAZY);
		ret = dlsym(handle, (const char *)procname);
	}

	pkt = packet_begin(FUNC_GLXGETPROCADDRESS);
	packet_write_string(pkt, (const char *)procname);
	packet_write_pointer(pkt, ret);
	packet_send(pkt, get_out_fd());

	return ret;
}

void (*glXGetProcAddressARB(const GLubyte *))(void) __attribute__((alias("glXGetProcAddress")));

#include "gensrc/glwrap.funcs"
