/* $Id$

This file is part of gldbg
Copyright © 2010  Mikko Rasa, Mikkosoft Productions
Distributed under the GPL
*/

#include <dlfcn.h>
#include "arraysize.h"
#include "breakpoint.h"
#include "functions.h"
#include "glwrap.h"
#include "opengl.h"
#include "packet.h"

GLenum cur_error = GL_NO_ERROR;

static void check_error()
{
	GLenum (*orig_glGetError)() = NULL;
	GLenum code;

	if(!orig_glGetError)
		orig_glGetError = glsym("glGetError");

	code = orig_glGetError();
	if(code!=GL_NO_ERROR)
	{
		GlPacket *pkt;

		pkt = packet_begin(FUNC_GLDERROR);
		packet_write_int(pkt, code);
		packet_send(pkt, get_out_fd());

		if(cur_error==GL_NO_ERROR)
			cur_error = code;
	}
}

GLenum APIENTRY glGetError()
{
	GLenum ret = cur_error;
	cur_error = GL_NO_ERROR;
	GlPacket *pkt;

	pkt = packet_begin(FUNC_GLGETERROR);
	packet_write_int(pkt, ret);
	packet_send(pkt, get_out_fd());

	return ret;
}

void (*eglGetProcAddress(const char *procname))(void)
{
	void *handle = NULL;
	void (*ret)() = NULL;
	GlPacket *pkt;

	if(glsym((const char *)procname))
	{
		handle = dlopen(NULL, RTLD_LAZY);
		ret = dlsym(handle, (const char *)procname);
	}

	pkt = packet_begin(FUNC_EGLGETPROCADDRESS);
	packet_write_pointer(pkt, ret);
	packet_write_string(pkt, (const char *)procname);
	packet_send(pkt, get_out_fd());

	return ret;
}

#include "gensrc/glwrap.funcs"
